# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - ItemSupport library

    @copyright: 2013 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMoin.util import lock
import os

# Content storage support.

class ItemDirectoryStore:

    "A directory-based item store."

    def __init__(self, path, lock_dir):

        "Initialise an item store for the given 'path' and 'lock_dir'."

        self.path = path
        self.next_path = os.path.join(self.path, "next")
        self.lock_dir = lock_dir
        self.writelock = lock.WriteLock(lock_dir)
        self.readlock = lock.ReadLock(lock_dir)

    def get_next(self):

        "Return the next item number."

        next = self.read_next()
        if next is None:
            next = self.deduce_next()
            self.write_next(next)
        return next

    def deduce_next(self):

        "Deduce the next item number from the existing item files."

        return max([int(filename) for filename in os.listdir(self.path) if filename.isdigit()] or [-1]) + 1

    def read_next(self):

        "Read the next item number from a special file."

        if not os.path.exists(self.next_path):
            return None

        f = open(self.next_path)
        try:
            try:
                return int(f.read())
            except ValueError:
                return None
        finally:
            f.close()

    def write_next(self, next):

        "Write the 'next' item number to a special file."

        f = open(self.next_path, "w")
        try:
            f.write(str(next))
        finally:
            f.close()

    def write_item(self, item, next):

        "Write the given 'item' to a file with the given 'next' item number."

        f = open(self.get_item_path(next), "w")
        try:
            f.write(item)
        finally:
            f.close()

    def read_item(self, number):

        "Read the item with the given item 'number'."

        f = open(self.get_item_path(number))
        try:
            return f.read()
        finally:
            f.close()

    def get_item_path(self, number):

        "Get the path for the given item 'number'."

        path = os.path.abspath(os.path.join(self.path, str(number)))
        basepath = os.path.join(self.path, "")

        if os.path.commonprefix([path, basepath]) != basepath:
            raise OSError, path

        return path

    # High-level methods.

    def append(self, item):

        "Append the given 'item' to the store."

        self.writelock.acquire()
        try:
            next = self.get_next()
            self.write_item(item, next)
            self.write_next(next + 1)
        finally:
            self.writelock.release()

    def __len__(self):

        """
        Return the number of the next item (which should also be the number of
        items).
        """

        self.writelock.acquire()
        try:
            return self.get_next()
        finally:
            self.writelock.release()

    def __iter__(self):

        "Return an iterator over the items in the store."

        return ItemIterator(self)

    def __getitem__(self, number):

        "Return the item with the given 'number'."

        self.readlock.acquire()
        try:
            try:
                return self.read_item(number)
            except (IOError, OSError):
                raise IndexError, number
        finally:
            self.readlock.release()

class ItemIterator:

    "An iterator over items in a store."

    def __init__(self, store, direction=1):
        self.store = store
        self.direction = direction
        self.reset()

    def reset(self):
        if self.direction == 1:
            self._next = 0
            self.final = len(self.store)
        else:
            self._next = len(self.store) - 1
            self.final = 0

    def more(self):
        if self.direction == 1:
            return self._next < self.final
        else:
            return self._next >= self.final

    def get_next(self):
        next = self._next
        self._next += self.direction
        return next

    def next(self):
        while self.more():
            try:
                return self.store[self.get_next()]
            except IndexError:
                pass

        raise StopIteration

    def reverse(self):
        self.direction = -self.direction
        self.reset()

    def reversed(self):
        self.reverse()
        return self

    def __iter__(self):
        return self

# vim: tabstop=4 expandtab shiftwidth=4
